/*=====================================================================
profiler.cpp
------------
File created by ClassTemplate on Thu Feb 24 19:00:30 2005

Copyright (C) Nicholas Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html..
=====================================================================*/
#include "profiler.h"


#include "../utils/stringutils.h"
#include <process.h>
#include <iostream>
#include <assert.h>
#include <winnt.h>


Profiler::Profiler(HANDLE target_thread_)
:	target_thread(target_thread_)
{
	
}


Profiler::~Profiler()
{
	
}


void Profiler::sampleTarget()
{
	// An open question is whther or not this routine can be called
	// reentrantly by the multi-media timer support.

	HRESULT result = SuspendThread(target_thread);

	if(result == 0xffffffff)
		throw ProfilerExcep("SuspendThread failed.");

	CONTEXT threadcontext;
	threadcontext.ContextFlags = CONTEXT_i386 | CONTEXT_CONTROL;
	result = GetThreadContext(target_thread, &threadcontext);
	if(!result)
		throw ProfilerExcep("GetThreadContext failed.");

	PROFILER_ADDR addr = threadcontext.Eip;

	//std::cout << "addr: " << addr << std::endl;

	result = ResumeThread(target_thread);
	if(!result)
		throw ProfilerExcep("ResumeThread failed.");

	//NOTE: this has to go after ResumeThread.  Otherwise mem allocation needed by std::map
	//may hit a lock held by the suspended thread.
	counts[addr]++;
}


void Profiler::saveIPs(std::ostream& stream)
{
	for(std::map<PROFILER_ADDR, int>::const_iterator i = counts.begin(); 
		i != counts.end(); ++i)
	{
		stream << ::toHexString((*i).first) << " " << (*i).second << "\n";
	}

	stream.flush();
}

