/*=====================================================================
profilerthread.cpp
------------------
File created by ClassTemplate on Thu Feb 24 19:29:41 2005

Copyright (C) Nicholas Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html..
=====================================================================*/
#include "profilerthread.h"


#include "../utils/stringutils.h"
#include <fstream>
#include <assert.h>
#include <algorithm>
#include "symbolinfo.h"

#pragma comment(lib, "winmm.lib")


void startProfiling(int num_samples)
{
	//get handle of the current thread
	/*const HANDLE thisthread = getProcessWideHandle();//OpenThread(THREAD_ALL_ACCESS, 0, GetCurrentThreadId());

	//------------------------------------------------------------------------
	//create profiler thread
	//------------------------------------------------------------------------
	ProfilerThread* profilerthread = new ProfilerThread(
											thisthread, //thread to target(this thread)
											num_samples, //do N samples
											false);//don't kill target when completed

	//------------------------------------------------------------------------
	//start the thread
	//------------------------------------------------------------------------
	profilerthread->launch();*/
}


const HANDLE getProcessWideHandle()
{
	HANDLE processWideThreadHandle;

	const bool result = DuplicateHandle(
							GetCurrentProcess(),
							GetCurrentThread(),
							GetCurrentProcess(),
							&processWideThreadHandle,
							0,
							FALSE, // not inheritable
							DUPLICATE_SAME_ACCESS);

	assert(result);
	return processWideThreadHandle;
}





ProfilerThread::ProfilerThread(HANDLE target_thread, DWORD target_process_id_/*, int num_samples*/, bool terminate_target_)
:	profiler(target_thread),
	target_process_id(target_process_id_),
	/*numsamples(num_samples),*/
	terminate_target(terminate_target_)
{
	numsamplessofar = 0;
}


ProfilerThread::~ProfilerThread()
{
	
}


void CALLBACK sprof_callback(UINT wTimerID, UINT msg,
						DWORD dwUser, DWORD dw1, DWORD dw2)
{
	ProfilerThread* thread = (ProfilerThread*)dwUser;

	thread->sample();

}

void ProfilerThread::sample()
{
	try
	{
		profiler.sampleTarget();
	}
	catch(ProfilerExcep& e)
	{
		error("ProfilerExcep: " + e.what());
		//------------------------------------------------------------------------
		//assume program has closed, kill this thread and display results.
		//------------------------------------------------------------------------
		//this->commit_suicide = true;
	}

	numsamplessofar++;
}

class ProcPred
{
public:
	bool operator () (std::pair<std::string, int>& a, std::pair<std::string, int>& b)
	{
		return a.second > b.second;
	}
};

void ProfilerThread::run()
{

	if(!SetThreadPriority(getHandle(), THREAD_PRIORITY_HIGHEST))
	{
		error("failed to set thread priority to highest.");
	}


	//there are two ways to do this... either to Sleep() 1 ms between samples,
	//or to use the Windows multimedia timer.
	//The Sleep method is simpler, and seems to work alright.

	const bool use_mm_timer = false;

	if(use_mm_timer)
	{
		HRESULT rc;

		HRESULT htimer = timeSetEvent(1, 1, sprof_callback, (DWORD)this,
		TIME_PERIODIC | TIME_CALLBACK_FUNCTION);
		assert(htimer);

		while(!this->commit_suicide)//while(numsamplessofar < numsamples)
		{
			Sleep(250);
		}

		rc = timeKillEvent(htimer);
		if(rc != TIMERR_NOERROR)
		{
			//error
		}
	}
	else
	{
		while(!this->commit_suicide)//numsamplessofar < numsamples)
		{
			Sleep(1);

			sample();
		}
	}



	std::cout << "computing and saving profile results..." << std::endl;

	std::map<std::string, int> proccounts;
	HANDLE process_handle = NULL;
	try
	{
		
		//get process id of the process the target thread is running in
		//const DWORD process_id = GetProcessIdOfThread(profiler.getTarget());

		//------------------------------------------------------------------------
		//Get handle to target process (handle is freed after this scope terminates)
		//------------------------------------------------------------------------
		process_handle = OpenProcess (PROCESS_ALL_ACCESS, FALSE, target_process_id); 

		//------------------------------------------------------------------------
		//load up the debug info for it
		//------------------------------------------------------------------------
		SymbolInfo syminfo(process_handle);

	
		//------------------------------------------------------------------------
		//save instruction pointer count results
		//------------------------------------------------------------------------
		std::ofstream outfile("IPcounts.txt");

		if(outfile)
		{
			//profiler.saveIPs(outfile);
			for(std::map<PROFILER_ADDR, int>::const_iterator i = profiler.counts.begin(); 
				i != profiler.counts.end(); ++i)
			{
				const PROFILER_ADDR addr = (*i).first;
				std::string file;
				int line;
				syminfo.getLineForAddr(addr, file, line);

				const int count = (*i).second;

				outfile << ::toHexString(addr) << " " << count << 
					"         \"" << file << "\" " << line << "\n";
			}

			outfile.flush();
		}
		else
		{
			error("failed to open 'IPcounts.txt' for writing");
		}

		//------------------------------------------------------------------------
		//resolve addresses to procedure names using the SymbolInfo class.
		//------------------------------------------------------------------------
		for(std::map<PROFILER_ADDR, int>::const_iterator i = profiler.counts.begin(); 
				i != profiler.counts.end(); ++i)
		{
			std::string procfile;
			int proclinenum;
			const std::string proc_name = "\"" + syminfo.getProcForAddr((*i).first, procfile, proclinenum) + "\"";

			const std::string full_proc_name = "\"" + syminfo.getModuleNameForAddr((*i).first) + "\" " + 
				proc_name + " \"" + procfile + "\"" + " " + ::toString(proclinenum);

			proccounts[full_proc_name] += (*i).second;
		}
	}
	catch(SymbolInfoExcep& e)
	{
		error("SymbolInfoExcep: " + e.what());
	}

	CloseHandle(process_handle);
	process_handle = NULL;

	//------------------------------------------------------------------------
	//copy proc counts map to vector so it can be sorted.
	//------------------------------------------------------------------------
	std::vector<std::pair<std::string, int> > proccounts_vec(proccounts.size());
	{
	int c = 0;
	for(std::map<std::string, int>::const_iterator i = proccounts.begin(); 
		i != proccounts.end(); ++i)
	{
		proccounts_vec[c].first = (*i).first;
		proccounts_vec[c].second = (*i).second;

		c++;
	}
	}

	//------------------------------------------------------------------------
	//sort procs by count
	//------------------------------------------------------------------------
	ProcPred pred;
	std::sort(proccounts_vec.begin(), proccounts_vec.end(), pred);

	//------------------------------------------------------------------------
	//calc percentages
	//------------------------------------------------------------------------
	std::vector<float> percentages(proccounts_vec.size());
	for(unsigned int i=0; i<proccounts_vec.size(); ++i)
		percentages[i] = (float)((double)proccounts_vec[i].second / (double)numsamplessofar
						 * 100.0);

	//------------------------------------------------------------------------
	//write proc counts to disk
	//------------------------------------------------------------------------
	std::ofstream namefile("ProcCounts.txt");

	if(namefile)
	{
		for(int z = 0; z != proccounts_vec.size(); ++z)
		{
			const std::string procname = proccounts_vec[z].first;//demangleProcName(proccounts_vec[z].first);
			namefile << procname;

			const int COL2_OFFSET = 120;//this is how many chars over from the left margin the count numbers are

			if(procname.size() <= COL2_OFFSET)
				for(unsigned int t=0; t<(COL2_OFFSET - procname.size()); ++t)
					namefile << " ";

			namefile << " " << proccounts_vec[z].second;

			//namefile << "  (" << ::floatToString(percentages[z], 2) << "%)\n";
			namefile << "  " << ::floatToString(percentages[z], 2) << "\n";
		}

		namefile.flush();
	}
	else
	{
		error("failed to open 'ProcCounts.txt' for writing.");
		return;
	}



	std::cout << "Profiling successfully completed." << std::endl;
	//::MessageBox(NULL, "Profiling successfully completed.", "Profiler", MB_OK);

	if(terminate_target)
	{
		std::cout << "Terminating target thread..." << std::endl;

		TerminateThread(profiler.getTarget(), 1);
	}

}


void ProfilerThread::error(const std::string& what)
{
	std::cerr << "ProfilerThread Error: " << what << std::endl;

	::MessageBox(NULL, std::string("Error: " + what).c_str(), "Profiler Error", MB_OK);
}
