/*=====================================================================
mainwin.cpp
-----------
File created by ClassTemplate on Sun Mar 13 21:12:40 2005

Copyright (C) Nicholas Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html.
=====================================================================*/
#include "mainwin.h"

#include "../utils/stringutils.h"

// IDs for the controls and the menu commands
enum
{
    // menu items
    Minimal_Quit = 1,

    // it is important for the id corresponding to the "About" command to have
    // this standard value as otherwise it won't be handled properly under Mac
    // (where it is special and put into the "Apple" menu)
    Minimal_About = wxID_ABOUT
};


MainWin::MainWin(const wxString& title, const std::string& procpath, const std::string& ipcountspath,
			const wxPoint& pos, const wxSize& size,
            long style)
:	wxFrame(NULL, -1, title, pos, size, style)
{
	panel = NULL;
	proclist = NULL;
	sourceview = NULL;

	    // set the frame icon
    SetIcon(wxICON(mondrian));

#if wxUSE_MENUS
    // create a menu bar
    wxMenu *menuFile = new wxMenu;

    // the "About" item should be in the help menu
    wxMenu *helpMenu = new wxMenu;
    helpMenu->Append(Minimal_About, _T("&About...\tF1"), _T("Show about dialog"));

    menuFile->Append(Minimal_Quit, _T("E&xit\tAlt-X"), _T("Quit this program"));

    // now append the freshly created menu to the menu bar...
    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(menuFile, _T("&File"));
    menuBar->Append(helpMenu, _T("&Help"));

    // ... and attach this menu bar to the frame
    SetMenuBar(menuBar);
#endif // wxUSE_MENUS

#if wxUSE_STATUSBAR
    // create a status bar just for fun (by default with 1 pane only)
	CreateStatusBar(2);
	SetStatusText(_T("Welcome to wxWindows!"));
#endif // wxUSE_STATUSBAR

	panel = new wxPanel(this, -1);

	sourceview = new SourceView(panel, this);

	//------------------------------------------------------------------------
	//Create list control
	//------------------------------------------------------------------------
	proclist = new ProcList(panel, LIST_CTRL,
                            wxDefaultPosition, wxDefaultSize,
                            /*flags |*/
                            wxSUNKEN_BORDER | wxLC_EDIT_LABELS, 
							sourceview);

	proclist->loadFromPath(procpath, ipcountspath);

	

	//sourceview->showFile("C:\\programming\\wxWindows-2.4.2\\src\\jpeg\\jcapimin.c", 1);

	SendSizeEvent();
}


MainWin::~MainWin()
{
	
}


void MainWin::OnSize(wxSizeEvent& event)
{
    wxSize size = GetClientSize();
    wxCoord y = (1*size.y)/3;
	if(proclist)
		proclist->SetSize(0, 0, size.x, y);

	if(sourceview)
		sourceview->SetSize(0, y + 1, size.x, size.y - y);

    event.Skip();
}


// the event tables connect the wxWindows events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(MainWin, wxFrame)
    EVT_SIZE(MainWin::OnSize)
	EVT_PAINT(MainWin::OnPaint)

    EVT_MENU(Minimal_Quit,  MainWin::OnQuit)
    EVT_MENU(Minimal_About, MainWin::OnAbout)	
END_EVENT_TABLE()

void MainWin::OnPaint(wxCommandEvent& event)
{
	//TEMP HACK:
	/*if(sourceview)
	{
		SetStatusText(sourceview->getCurrentFile().c_str(), 0);
		SetStatusText(::toString(sourceview->GetInsertionPoint()).c_str(), 1);
	}*/


	event.Skip();
}

void MainWin::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    // TRUE is to force the frame to close
    Close(TRUE);
}

void MainWin::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    wxString msg;
    msg.Printf( _T("Sleepy v0.1\n")
				_T("Code by Nicholas Chapman, 2005\n")
				_T("http://sleepy.sourceforge.net/\n")
				_T("nickamy@paradise.net.nz\n")
                _T("Uses %s"), wxVERSION_STRING);

    wxMessageBox(msg, _T("About Profiler GUI"), wxOK | wxICON_INFORMATION, this);
}

void MainWin::setCurrent(const std::string& currentfile_, int currentline_)
{
	if(currentfile != currentfile_ || currentline != currentline_)
	{
		currentfile = currentfile_;
		currentline = currentline_;

		SetStatusText(std::string("source file: " + currentfile).c_str(), 0);
		SetStatusText(std::string("line " + ::toString(currentline)).c_str(), 1);
	}
}






