/*=====================================================================
proclist.cpp
------------
File created by ClassTemplate on Tue Mar 15 21:13:18 2005

Copyright (C) Nicholas Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html.
=====================================================================*/
#include "proclist.h"

#include "../utils/stringutils.h"
#include "sourceview.h"
#include <fstream>
#include "globals.h"

BEGIN_EVENT_TABLE(ProcList, wxListCtrl)
	EVT_LIST_ITEM_SELECTED(LIST_CTRL, ProcList::OnSelected)
END_EVENT_TABLE()

ProcList::ProcList(wxWindow *parent, const wxWindowID id, const wxPoint& pos,
               const wxSize& size, long style, SourceView* sourceview_)
:	wxListCtrl(parent, id, pos, size, wxLC_REPORT /*style*/),
    m_attr(*wxBLUE, *wxLIGHT_GREY, wxNullFont),
	sourceview(sourceview_)
{
	wxListItem itemCol;
    itemCol.m_mask = wxLIST_MASK_TEXT | wxLIST_MASK_IMAGE;
    itemCol.m_text = _T("Procedure");
    itemCol.m_image = -1;
    InsertColumn(0, itemCol);
	SetColumnWidth(0, 120);

    itemCol.m_text = _T("module");
    InsertColumn(1, itemCol);

    itemCol.m_text = _T("source file");
	InsertColumn(2, itemCol);
	SetColumnWidth(2, 245);

	itemCol.m_text = _T("source line");
    InsertColumn(3, itemCol);

	itemCol.m_text = _T("count");
    InsertColumn(4, itemCol);

	itemCol.m_text = _T("Percentage");
    InsertColumn(5, itemCol);
}


ProcList::~ProcList()
{
	
}

void ProcList::OnSelected(wxListEvent& event)
{
    assert(GetWindowStyle() & wxLC_REPORT);

	wxListItem info;
    info.m_itemId = event.m_itemIndex;
    info.m_col = 2;
    info.m_mask = wxLIST_MASK_TEXT;
    if ( GetItem(info) )
    {
		//pull out the source file name
		const std::string srcpath = info.m_text.c_str();

		//pull out the line number where the proc is defined
		int linenum;
		info.m_col = 3;
		GetItem(info);
		linenum = ::stringToInt(info.m_text.c_str());

		if(fileinfo.find(srcpath) != fileinfo.end())
		{
			sourceview->showFile(srcpath, linenum, fileinfo[srcpath]);
		}
		else
		{
			LINEINFOMAP dummymap;
			sourceview->showFile("[no info]", 0, dummymap);
		}
	}
	else
	{
		wxFAIL_MSG(wxT("wxListCtrl::GetItem() failed"));
	}
}

void ProcList::loadFromPath(const std::string& proclistpath, const std::string& ipcountspath)
{
	{
	std::ifstream file(proclistpath.c_str());

	if(!file)
	{
		::fatalError("could not open file '" + proclistpath + "'.");
	}


	int c = 0;
	while(file)
	{
		std::string line;
		std::getline(file, line);
		
		if(!file)
			break;

		std::istringstream stream(line);

		std::string module;
		::readQuote(stream, module);

		std::string procname;
		::readQuote(stream, procname);

		std::string sourcefile;
		::readQuote(stream, sourcefile);

		int linenum;
		stream >> linenum;

		int count;
		stream >> count;

		float percentage;
		stream >> percentage;


		long tmp = this->InsertItem(c, procname.c_str(), -1);
		SetItemData(tmp, c);
		this->SetItem(c, 1, module.c_str());
		this->SetItem(c, 2, sourcefile.c_str());
		this->SetItem(c, 3, ::toString(linenum).c_str());
		this->SetItem(c, 4, ::toString(count).c_str());
		this->SetItem(c, 5, ::toString(percentage).c_str());


		c++;
	}

	}


	//------------------------------------------------------------------------
	//go thru once to get the total instruction count
	//------------------------------------------------------------------------
	int totalcount = 0;
	{
	std::ifstream file(ipcountspath.c_str());
	
	if(!file)
	{
		::fatalError("could not open file '" + ipcountspath + "'.");
	}

	while(file)
	{
		std::string line;
		std::getline(file, line);
		
		if(!file)
			break;

		std::istringstream stream(line);

		std::string memaddr;
		stream >> memaddr;
		int count;
		stream >> count;
		totalcount += count;
	}

	}

	{
	std::ifstream file(ipcountspath.c_str());
	assert(file);

	//reset file read pointer
	//file.seekg( 0, ios_base::beg );

	int c = 0;
	while(file)
	{
		std::string line;
		std::getline(file, line);
		
		if(!file)
			break;

		std::istringstream stream(line);

		std::string memaddr;
		stream >> memaddr;

		int count;
		stream >> count;

		std::string srcfile;
		::readQuote(stream, srcfile);

		int linenum;
		stream >> linenum;

		LineInfo& lineinfo = (fileinfo[srcfile])[linenum];
		lineinfo.count += count;
		lineinfo.percentage += 100.0f * ((float)count / (float)totalcount);
	}

	}

}





